@@ -78,7 +78,7 @@ pool <- function(y, er, deltaR=0, deltaSTD=0, threshold=.05, roundby=1) {
 #'   Zu <- grep("ETH", shroud$ID) # Zurich lab only
 #'   as.one(shroud$y[Zu],shroud$er[Zu], BCAD=TRUE)
 #' @export
-as.one <- function(y, er, cc=1, postbomb=FALSE, deltaR=0, deltaSTD=0, is.F=FALSE, as.F=FALSE, thiscurve=NULL, yrsteps=1, threshold=1e-3, normal=TRUE, t.a=3, t.b=4, BCAD=FALSE, cc.dir=NULL, age.lim=c(), age.lab=c(), d.lim=c(), calib.col=rgb(0,0,0,.2), one.col=rgb(0,0,1,.5), one.height=4, prob=0.95, talk=TRUE, roundby=0, bty="n") {
+as.one <- function(y, er, cc=1, postbomb=FALSE, deltaR=0, deltaSTD=0, is.F=FALSE, as.F=FALSE, thiscurve=NULL, yrsteps=1, threshold=1e-3, normal=TRUE, t.a=3, t.b=4, BCAD=FALSE, cc.dir=NULL, age.lim=c(), age.lab=c(), d.lim=c(), calib.col=rgb(0,0,0,.2), one.col=rgb(0,0,1,.5), one.height=.3, prob=0.95, talk=TRUE, roundby=0, bty="n") {
 
   y <- y - deltaR
   er <- sqrt(er^2 + deltaSTD^2)
@@ -101,7 +101,7 @@ as.one <- function(y, er, cc=1, postbomb=FALSE, deltaR=0, deltaSTD=0, is.F=FALSE
   product <- apply(probs, 2, prod)
 
   as.dist <- cbind(xseq, product/max(product))
-  hpds <- draw.dist(as.dist, y.pos=max(d.lim), prob=prob, dist.col=one.col, ex=one.height)
+  hpds <- draw.dist(as.dist, y.pos=max(d.lim), prob=prob, dist.col=one.col, fraction=one.height, as.unit=FALSE)
   if(talk) {
     as.points <- suppressWarnings(point.estimates(as.dist, rounded=roundby))
     message("point estimates (mean, median, mode and midpoint): ", as.points[1], ", ", as.points[2], ", ", as.points[3], " & ", as.points[4], ifelse(BCAD, " BC/AD", " cal BP"))
@@ -159,7 +159,7 @@ as.one <- function(y, er, cc=1, postbomb=FALSE, deltaR=0, deltaSTD=0, is.F=FALSE
 #'   # bins of 50 yr, moving by 10 yr, slow
 #' }
 #' @export
-as.bin <- function(y, er, width=100, move.by=c(), move.res=100, cc=1, postbomb=FALSE, deltaR=0, deltaSTD=0, is.F=FALSE, as.F=FALSE, thiscurve=NULL, yrsteps=1, threshold=1e-3, normal=TRUE, t.a=3, t.b=4, BCAD=FALSE, cc.dir=NULL, age.lim=c(), age.lab=c(), d.lim=c(), calib.col=rgb(0,0,0,.2), bin.col=rgb(0,0,1,.5), bin.height=4, talk=TRUE, prob=0.95, roundby=0, bty="n") {
+as.bin <- function(y, er, width=100, move.by=c(), move.res=100, cc=1, postbomb=FALSE, deltaR=0, deltaSTD=0, is.F=FALSE, as.F=FALSE, thiscurve=NULL, yrsteps=1, threshold=1e-3, normal=TRUE, t.a=3, t.b=4, BCAD=FALSE, cc.dir=NULL, age.lim=c(), age.lab=c(), d.lim=c(), calib.col=rgb(0,0,0,.2), bin.col=rgb(0,0,1,.5), bin.height=.3, talk=TRUE, prob=0.95, roundby=0, bty="n") {
 
   y <- y - deltaR
   er <- sqrt(er^2 + deltaSTD^2)
@@ -181,7 +181,7 @@ as.bin <- function(y, er, width=100, move.by=c(), move.res=100, cc=1, postbomb=F
 
   inbin <- rowSums(tmp)
   as.dist <- cbind(xseq, inbin/max(inbin))
-  hpds <- draw.dist(as.dist, y.pos=max(d.lim), prob=prob, dist.col=bin.col, ex=bin.height)
+  hpds <- draw.dist(as.dist, y.pos=max(d.lim), prob=prob, dist.col=bin.col, as.unit=FALSE, fraction=bin.height)
   
   if(talk) {
     as.points <- suppressWarnings(point.estimates(as.dist, rounded=roundby))
@@ -198,7 +198,7 @@ as.bin <- function(y, er, width=100, move.by=c(), move.res=100, cc=1, postbomb=F
 
 
 
-#' @name overlap
+#' @name overlapping
 #' @title The overlap between calibrated C14 dates
 #' @description Calculates the amount of overlap (as percentage) between two or more calibrated radiocarbon dates. It does this by taking a sequence of calendar dates 'x' and for each calendar date find the calibrated distribution with the minimum height - this minimum height is taken as the overlap between the dates for that age. This is repeated for all 'x'. The sum of these heights is the overlap, which can reach values from 0 to 100\%. 
 #' @return The overlap between all calibrated probabilities as percentage, and a plot. 
@@ -229,9 +229,9 @@ as.bin <- function(y, er, width=100, move.by=c(), move.res=100, cc=1, postbomb=F
 #' @examples
 #'   y <- c(3820, 4430) # the C14 ages of a twig and a marine shell from a single layer
 #'   er <- c(40, 40) # their lab errors
-#'   overlap(y, er, cc=1:2) 
+#'   overlapping(y, er, cc=1:2) 
 #' @export
-overlap <- function(y, er, res=1e3, cc=1, postbomb=FALSE, deltaR=0, deltaSTD=0, thiscurve=NULL, BCAD=FALSE, normal=TRUE, t.a=3, t.b=4, cc.dir=NULL, threshold=0.001, age.lim=c(), age.lab=c(), calib.col=rgb(0,0,0,.2), overlap.col=rgb(0,0,1,.4), overlap.border=NA, overlap.height=1, talk=TRUE, prob=0.95, roundby=1, bty="n") {
+overlapping <- function(y, er, res=1e3, cc=1, postbomb=FALSE, deltaR=0, deltaSTD=0, thiscurve=NULL, BCAD=FALSE, normal=TRUE, t.a=3, t.b=4, cc.dir=NULL, threshold=0.001, age.lim=c(), age.lab=c(), calib.col=rgb(0,0,0,.2), overlap.col=rgb(0,0,1,.4), overlap.border=NA, overlap.height=1, talk=TRUE, prob=0.95, roundby=1, bty="n") {
  
   y <- y - deltaR
   er <- sqrt(er^2 + deltaSTD^2)	
@@ -281,7 +281,7 @@ dist.overlap <- function(dist1, dist2, by=1, visualise=TRUE, prob=0.95, BCAD=FAL
       if(BCAD)
         xlab <- "BC/AD" else
           xlab <- "cal BP"
-	  
+
     plot(0, type="n", xlim=xrng, xlab=xlab, ylim=c(0, -3.3), ylab="", bty=bty, yaxt=yaxt)	
     draw.dist(cbind(xseq, prob1), dist.col=dist1.col, dist.border=dist1.col, prob=prob, mirror=FALSE, y.pos=-2.2, peak=T)
     draw.dist(cbind(xseq, prob2), dist.col=dist2.col, dist.border=dist2.col, prob=prob, y.pos=-1.1, peak=T)
@@ -295,11 +295,11 @@ dist.overlap <- function(dist1, dist2, by=1, visualise=TRUE, prob=0.95, BCAD=FAL
 
 #' @name spread
 #' @title The spread among calibrated dates
-#' @description Calculates the spread among multiple calibrated radiocarbon dates. It does this by randomly sampling ages from the calibrated dates, and calculate the difference between one random date and all others for that iteration. 
+#' @description Calculates the spread among multiple calibrated radiocarbon dates. It does this by randomly sampling ages from the calibrated dates, and calculating the difference between one random date and all others for that iteration.
 #' @return The spread of all calibrated probabilities. 
 #' @param y The set of radiocarbon dates
 #' @param er The lab errors of the radiocarbon dates
-#' @param n The number of iterations to base the calculations on. Defaults to 100,000.
+#' @param n The number of iterations to base the calculations on. Defaults to 100,000. Different values for n could significantly alter performance and accuracy.
 #' @param cc Calibration curve to use. Defaults to IntCal20 (\code{cc=1}).
 #' @param postbomb Whether or not to use a postbomb curve. Required for negative radiocarbon ages.
 #' @param deltaR Age offset (e.g. for marine samples).
@@ -337,9 +337,8 @@ spread <- function(y, er, n=1e5, cc=1, postbomb=FALSE, deltaR=0, deltaSTD=0, as.
     xs[,i] <- r.calib(n, y[i], er[i], cc=cc, postbomb=postbomb, as.F=as.F, thiscurve=thiscurve, yrsteps=yrsteps, cc.resample=cc.resample, threshold=threshold, normal=normal, t.a=t.a, t.b=t.b, cc.dir=cc.dir)
   for(i in 1:n)
     diffs[i,] <- abs(xs[i,ns[i]] - xs[i,-ns[i]])
-  
   as.dens <- density(diffs)
-  as.hpd <- hpd(diffs)
+  as.hpd <- hpd(diffs, every=min(diff(as.dens$x)))
   aspoints <- round(point.estimates(cbind(as.dens$x, as.dens$y)), roundby)
   oneprob <- (1-prob)/2
   minmax <- round(quantile(diffs, probs=c(oneprob, .5, 1-oneprob)), roundby)
@@ -357,3 +356,94 @@ spread <- function(y, er, n=1e5, cc=1, postbomb=FALSE, deltaR=0, deltaSTD=0, as.
   
   invisible(diffs)
 }
+
+
+
+#' @name span
+#' @title The time span between two calibrated dates
+#' @description Calculates the timespan between two calibrated radiocarbon dates. It does this by randomly sampling ages from both calibrated dates, followed by calculating the differences between all samples ages.
+#' @return The time span.
+#' @param y1 The first radiocarbon date.
+#' @param er1 The lab error of the first radiocarbon date.
+#' @param y2 The second radiocarbon date.
+#' @param er2 The lab error of the second radiocarbon date.
+#' @param n The number of iterations to base the calculations on. Defaults to 100,000. Different values for n could significantly alter performance and accuracy.
+#' @param positive Whether or not to enforce the span to be positive. If set to TRUE, then negative span values are removed. Defaults to FALSE.
+#' @param cc Calibration curve(s) to use. Defaults to IntCal20 (\code{cc=1}). Can be a vector of length 2.
+#' @param postbomb Whether or not to use a postbomb curve. Required for negative radiocarbon ages.
+#' @param deltaR Age offset (e.g. for marine samples). Can be a vector of length 2.
+#' @param deltaSTD Uncertainty of the age offset (1 standard deviation). Can be a vector of length 2.
+#' @param as.F Whether or not to calculate ages in the F14C realm. Defaults to \code{as.F=FALSE}, which uses the C14 realm.
+#' @param thiscurve As an alternative to providing cc and/or postbomb, the data of a specific curve can be provided (3 columns: cal BP, C14 age, error).
+#' @param yrsteps Steps to use for interpolation. Defaults to the cal BP steps in the calibration curve
+#' @param cc.resample The IntCal20 curves have different densities (every year between 0 and 5 kcal BP, then every 5 yr up to 15 kcal BP, then every 10 yr up to 25 kcal BP, and then every 20 yr up to 55 kcal BP). If calibrated ages span these density ranges, their drawn heights can differ, as can their total areas (which should ideally all sum to the same size). To account for this, resample to a constant time-span, using, e.g., cc.resample=5 for 5-yr timespans.
+#' @param threshold Report only values above a threshold. Defaults to \code{threshold=1e-6}.
+#' @param normal Use the normal distribution to calibrate dates (default TRUE). The alternative is to use the t model (Christen and Perez 2016).
+#' @param t.a Value a of the t distribution (defaults to 3). Can be a vector of length 2.
+#' @param t.b Value b of the t distribution (defaults to 4). Can be a vector of length 2.
+#' @param cc.dir Directory of the calibration curves. Defaults to where the package's files are stored (system.file), but can be set to, e.g., \code{cc.dir="curves"}.
+#' @param visualise Whether or not to plot the time span.
+#' @param talk Whether or not to report a summary of the span.
+#' @param prob Probability range to report. Defaults to \code{prob=0.95}.
+#' @param roundby Number of decimals to report
+#' @param bty Draw a box around a box of a certain shape. Defaults to \code{bty="l"}.
+#' @examples
+#'   span(2300, 20, 2350, 20)
+#' @export
+span <- function(y1, er1, y2, er2, n=1e5, positive=FALSE, cc=1, postbomb=FALSE, deltaR=0, deltaSTD=0, as.F=FALSE, thiscurve=NULL, yrsteps=1, cc.resample=FALSE, threshold=1e-3, normal=TRUE, t.a=3, t.b=4, cc.dir=NULL, visualise=TRUE, talk=TRUE, prob=0.95, roundby=1, bty="l") {
+
+  if(!length(y1) == 1)
+    stop("y1 must be of length 1")
+  if(!length(er1) == 1)
+    stop("er1 must be of length 1")
+  if(!length(y2) == 1)
+    stop("y2 must be of length 1")
+  if(!length(er2) == 1)
+    stop("er2 must be of length 1")
+  if(length(cc) == 1)
+    cc <- c(cc, cc)
+  if(length(t.a) == 1)
+    t.a <- c(t.a, t.a)
+  if(length(t.b) == 1)
+    t.b <- c(t.b, t.b)
+  if(length(deltaSTD) == 1)
+    deltaSTD <- c(deltaSTD, deltaSTD)
+  if(length(deltaR) == 1)
+    deltaR <- c(deltaR, deltaR)
+  if(n < 100)
+    message("very few iterations, results may be unreliable")
+  if(n > 1e6)
+    message("very many iterations, overkill, this may take a while and be difficult to compute")
+
+  y1 <- y1 - deltaR[1]
+  er1 <- sqrt(er1^2 + deltaSTD[1]^2)
+  y2 <- y2 - deltaR[2]
+  er2 <- sqrt(er2^2 + deltaSTD[2]^2)
+
+  x1 <- r.calib(n, y1, er1, cc=cc[1], postbomb=postbomb, as.F=as.F, thiscurve=thiscurve, yrsteps=yrsteps, cc.resample=cc.resample, threshold=threshold, normal=normal, t.a=t.a[1], t.b=t.b[1], cc.dir=cc.dir)
+  x2 <- r.calib(n, y2, er2, cc=cc[2], postbomb=postbomb, as.F=as.F, thiscurve=thiscurve, yrsteps=yrsteps, cc.resample=cc.resample, threshold=threshold, normal=normal, t.a=t.a[2], t.b=t.b[2], cc.dir=cc.dir)
+  diffs <- x2 - x1
+  if(positive) {
+    diffs <- diffs[which(diffs >= 0)]
+    as.dens <- density(diffs, from=0)
+    } else
+      as.dens <- density(diffs)
+
+  as.hpd <- hpd(cbind(as.dens$x, as.dens$y))
+  aspoints <- round(point.estimates(cbind(as.dens$x, as.dens$y)), roundby)
+  oneprob <- (1-prob)/2
+  minmax <- round(quantile(diffs, probs=c(oneprob, .5, 1-oneprob)), roundby)
+  if(talk)
+    message("average span: ", aspoints[1], " calendar years (",
+      "median ", aspoints[2], ")\n95% range: ", minmax[1], " to ", minmax[3])
+
+  if(visualise) {
+    plot(as.dens, xlab="span (calendar years)", main="", bty=bty)
+    segments(minmax[1], 0, minmax[3], 0, lwd=4, col=rgb(0,0,0,.5), lend=1)
+    abline(v=c(mean(diffs), minmax[2]), col=c(2,4), lty=2)
+    legend("topright", legend=c("span", "range", "median", "mean"),
+      text.col=c(1,grey(.5), 4, 2), bty="n", cex=.7)
+  }
+
+  invisible(diffs)
+}
